# -*- coding: utf-8 -*-

from openfisca_senegal.base import *
from numpy import clip, floor_divide


class date_de_naissance(Variable):
    column = DateCol
    entity = Individu
    label = u"Date de naissance"


class salaire(Variable):
    column = FloatCol
    entity = Individu
    label = "Salaire"
    set_input = set_input_divide_by_period


class est_marie(Variable):
    column = BoolCol
    entity = Individu
    label = u"Est marié"
    set_input = set_input_dispatch_by_period


class est_divorce(Variable):
    column = BoolCol
    entity = Individu
    label = u"Est divorcé"
    set_input = set_input_dispatch_by_period


class est_veuf(Variable):
    column = BoolCol
    entity = Individu
    label = u"Est veuf"
    set_input = set_input_dispatch_by_period


class est_celibataire(Variable):
    column = BoolCol
    entity = Individu
    label = u"Est célibataire"
    set_input = set_input_dispatch_by_period


class conjoint_a_des_revenus(Variable):
    column = BoolCol
    entity = Individu


class nombre_enfants(Variable):
    column = IntCol
    entity = Individu


class pension_retraite(Variable):
    column = FloatCol
    entity = Individu
    label = u"Pension Retraite"
    set_input = set_input_divide_by_period


class benefices_non_salarie(Variable):
    column = FloatCol
    entity = Individu
    label = u"Bénéfices non salarié"
    set_input = set_input_divide_by_period


class nombre_de_parts(Variable):
    column = FloatCol
    entity = Individu
    label = u"Nombre de parts"

    def function(individu, period):
        nombre_de_parts_enfants = individu('nombre_enfants', period) * 0.5
        conjoint_a_des_revenus = individu('conjoint_a_des_revenus', period)
        est_marie = individu('est_marie', period)
        nombre_de_parts_conjoint = est_marie * (0.5 + not_(conjoint_a_des_revenus) * 0.5)

        nombre_de_parts = 1 + nombre_de_parts_conjoint + nombre_de_parts_enfants

        return period, min_(5, nombre_de_parts)


class impot_avant_reduction_famille(Variable):
    column = FloatCol
    entity = Individu

    def function(individu, period, legislation):
        salaire = individu('salaire', period, options = [ADD])
        salaire_abattement = min_(0.3 * salaire, 900000)
        salaire_imposable = salaire - salaire_abattement

        pension_retraite = individu('pension_retraite', period, options = [ADD])
        pension_abbattement = max_(pension_retraite * 0.4, 1800000) * (pension_retraite > 0)
        retraite_imposable = pension_retraite - pension_abbattement
        benefices_non_salarie = individu('benefices_non_salarie', period, options = [ADD])
        benefice_abattement = benefices_non_salarie * 0.15
        benefices_imposable = benefices_non_salarie - benefice_abattement

        revenus_arrondis = floor_divide(salaire_imposable + retraite_imposable + benefices_imposable, 1000) * 1000
        revenus_imposable = max_(0, revenus_arrondis)

        bareme_impot_progressif = legislation(period).bareme_impot_progressif
        return period, bareme_impot_progressif.calc(revenus_imposable)


class reduction_impots_pour_charge_famille(Variable):
    column = FloatCol
    entity = Individu

    def function(individu, period, legislation):
        impot_avant_reduction_famille = individu('impot_avant_reduction_famille', period)

        nombre_de_parts = individu('nombre_de_parts', period)
        reductions_pour_charge_de_famille = legislation(period).reductions_pour_charge_de_famille

        taux = (nombre_de_parts == 1) * reductions_pour_charge_de_famille.taux_1 + \
            (nombre_de_parts == 1.5) * reductions_pour_charge_de_famille.taux_2 + \
            (nombre_de_parts == 2) * reductions_pour_charge_de_famille.taux_3 + \
            (nombre_de_parts == 2.5) * reductions_pour_charge_de_famille.taux_4 + \
            (nombre_de_parts == 3) * reductions_pour_charge_de_famille.taux_5 + \
            (nombre_de_parts == 3.5) * reductions_pour_charge_de_famille.taux_6 + \
            (nombre_de_parts == 4) * reductions_pour_charge_de_famille.taux_7 + \
            (nombre_de_parts == 4.5) * reductions_pour_charge_de_famille.taux_8 + \
            (nombre_de_parts == 5) * reductions_pour_charge_de_famille.taux_9
        minimum = (nombre_de_parts == 1) * reductions_pour_charge_de_famille.min_1 + \
            (nombre_de_parts == 1.5) * reductions_pour_charge_de_famille.min_2 + \
            (nombre_de_parts == 2) * reductions_pour_charge_de_famille.min_3 + \
            (nombre_de_parts == 2.5) * reductions_pour_charge_de_famille.min_4 + \
            (nombre_de_parts == 3) * reductions_pour_charge_de_famille.min_5 + \
            (nombre_de_parts == 3.5) * reductions_pour_charge_de_famille.min_6 + \
            (nombre_de_parts == 4) * reductions_pour_charge_de_famille.min_7 + \
            (nombre_de_parts == 4.5) * reductions_pour_charge_de_famille.min_8 + \
            (nombre_de_parts == 5) * reductions_pour_charge_de_famille.min_9
        maximum = (nombre_de_parts == 1) * reductions_pour_charge_de_famille.max_1 + \
            (nombre_de_parts == 1.5) * reductions_pour_charge_de_famille.max_2 + \
            (nombre_de_parts == 2) * reductions_pour_charge_de_famille.max_3 + \
            (nombre_de_parts == 2.5) * reductions_pour_charge_de_famille.max_4 + \
            (nombre_de_parts == 3) * reductions_pour_charge_de_famille.max_5 + \
            (nombre_de_parts == 3.5) * reductions_pour_charge_de_famille.max_6 + \
            (nombre_de_parts == 4) * reductions_pour_charge_de_famille.max_7 + \
            (nombre_de_parts == 4.5) * reductions_pour_charge_de_famille.max_8 + \
            (nombre_de_parts == 5) * reductions_pour_charge_de_famille.max_9

        reduction_impot = clip(impot_avant_reduction_famille * taux, a_min = minimum, a_max = maximum)
        return period, reduction_impot


class impot_revenus(Variable):
    column = FloatCol
    entity = Individu

    def function(individu, period):
        impot_avant_reduction_famille = individu('impot_avant_reduction_famille', period)
        reduction_impots_pour_charge_famille = individu('reduction_impots_pour_charge_famille', period)
        impot_apres_reduction_famille = impot_avant_reduction_famille - reduction_impots_pour_charge_famille
        return period, max_(0, impot_apres_reduction_famille)





# -*- coding: utf-8 -*-

import os

from openfisca_core.taxbenefitsystems import TaxBenefitSystem

from . import entities, scenarios


COUNTRY_DIR = os.path.dirname(os.path.abspath(__file__))
MODEL_FILE = os.path.join(COUNTRY_DIR, 'model.py')
PARAM_FILE = os.path.join(COUNTRY_DIR, 'parameters.xml')


class SenegalTaxBenefitSystem(TaxBenefitSystem):
    """Senegalese tax and benefit system"""
    CURRENCY = u"FCFA"

    def __init__(self):
        super(SenegalTaxBenefitSystem, self).__init__(entities.entities)
        self.Scenario = scenarios.Scenario
        self.add_variables_from_file(MODEL_FILE)
        self.add_legislation_params(PARAM_FILE)



# -*- coding: utf-8 -*-


from openfisca_core import conv
from openfisca_core.scenarios import AbstractScenario


class Scenario(AbstractScenario):
    def init_single_entity(self, axes = None, enfants = None, famille = None, parent1 = None, parent2 = None,
            period = None):
        if enfants is None:
            enfants = []
        assert parent1 is not None
        famille = famille.copy() if famille is not None else {}
        individus = []
        for index, individu in enumerate([parent1, parent2] + (enfants or [])):
            if individu is None:
                continue
            id = individu.get('id')
            if id is None:
                individu = individu.copy()
                individu['id'] = id = 'ind{}'.format(index)
            individus.append(individu)
            if index <= 1:
                famille.setdefault('parents', []).append(id)
            else:
                famille.setdefault('enfants', []).append(id)
        famille.setdefault('enfants', [])
        conv.check(self.make_json_or_python_to_attributes())(dict(
            axes = axes,
            period = period,
            test_case = dict(
                familles = [famille],
                individus = individus,
                ),
            ))
        return self

    def make_json_or_python_to_test_case(self, period = None, repair = False):
        def json_or_python_to_test_case(value, state = None):
            return value, None



# -*- coding: utf-8 -*-

from datetime import date

from numpy import maximum as max_, minimum as min_, logical_not as not_, where, select

from openfisca_core.columns import (AgeCol, BoolCol, DateCol, EnumCol, FixedStrCol, FloatCol, IntCol,
    PeriodSizeIndependentIntCol, StrCol)
from openfisca_core.enumerations import Enum
from openfisca_core.formulas import (ADD,
    dated_function, DIVIDE, set_input_dispatch_by_period, set_input_divide_by_period)
from openfisca_core.variables import DatedVariable, Variable
from openfisca_core.formula_helpers import apply_thresholds, switch

from openfisca_senegal.entities import Famille, Individu

__all__ = [
    'ADD',
    'AgeCol',
    'apply_thresholds',
    'BoolCol',
    'date',
    'DateCol',
    'dated_function',
    'DatedVariable',
    'DIVIDE',
    'Enum',
    'EnumCol',
    'Famille',
    'FixedStrCol',
    'FloatCol',
    'Individu',
    'IntCol',
    'max_',
    'min_',
    'not_',
    'PeriodSizeIndependentIntCol',
    'select',
    'StrCol',
    'switch',
    'Variable',
    'where',
    'set_input_dispatch_by_period',
    'set_input_divide_by_period',
    ]



# -*- coding: utf-8 -*-


from openfisca_core.entities import build_entity


Famille = build_entity(
    key = "famille",
    plural = "familles",
    label = u'Famille',
    roles = [
        {
            'key': 'parent',
            'plural': 'parents',
            'label': u'Parents',
            },
        {
            'key': 'enfant',
            'plural': 'enfants',
            'label': u'Enfants',
            }
        ]
    )


Individu = build_entity(
    key = "individu",
    plural = "individus",
    label = u'Individu',
    is_person = True,
    )

entities = [Famille, Individu]
